package com.perforce.reviewer;

import java.util.*;
import java.io.*;
import java.text.*;
import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;
import com.perforce.api.*;

class MailListener implements ChangeListener {
  private Env env;
  private Session session = null;
  private String from = null;
  private String suffix = null;

  public MailListener() {
  }

  public void setEnv(Env env) {
    this.env = env;

    if (null == from) from = env.getProperty("p4.reviewer.from","nobody");
    if (null == suffix) suffix = env.getProperty("p4.reviewer.suffix");
    if (null == session) session = Session.getDefaultInstance(env.getProperties(), null);
  }

  public void handleChange(Change chng, User[] usrs) {
    Vector v = new Vector();
    int i;

    System.out.println("MailListener.handleChange: "+chng);

    // Let's only send to users within the company.
    for (i = 0; i < usrs.length; i++) {
      if (null == suffix || usrs[i].getEmail().endsWith(suffix)) {
	v.addElement(usrs[i]);
      }
    }
    
    //    session.setDebug(debug);
	
    try {
      // create a message
      Message msg = new MimeMessage(session);
      msg.setFrom(new InternetAddress(from));
      InternetAddress[] address = new InternetAddress[v.size()];
      for (i = 0; i < v.size(); i++) {
	address[i] = new InternetAddress(((User)v.elementAt(i)).getEmail());
      }
      User owner = chng.getUser();
      owner.sync();
      InternetAddress[] replyto = { new InternetAddress(owner.getEmail()) };
      msg.setRecipients(Message.RecipientType.TO, address);
      msg.setReplyTo(replyto);
      msg.setSubject("Change Review: "+chng.getNumber());
      msg.setSentDate(new Date());

      // If the desired charset is known, you can use
      // setText(text, charset)
      // msg.setText(chng.getDescription());


      // create and fill the first message part with text/plain version
      MimeBodyPart mbp1 = new MimeBodyPart();
      String sig = env.getProperty("p4.reviewer.sig.plain","");
      mbp1.setText(chng.getDescription()+"\n\n"+sig);

      // create and fill the second message part with text/html version
      MimeBodyPart mbp2 = new MimeBodyPart();
      mbp2.setDataHandler(getChangeData(chng));

      // create the Multipart and its parts to it
      Multipart mp = new MimeMultipart("alternative");
      mp.addBodyPart(mbp1);
      mp.addBodyPart(mbp2);

      // add the Multipart to the message
      msg.setContent(mp);
	    
      Transport.send(msg);
    } catch (MessagingException mex) {
      System.out.println("Reviewer: Trouble notifying users.\n");
      mex.printStackTrace(System.out);
      System.out.println();
      Exception ex = mex;
      do {
	if (ex instanceof SendFailedException) {
	  SendFailedException sfex = (SendFailedException)ex;
	  Address[] invalid = sfex.getInvalidAddresses();
	  if (invalid != null) {
	    System.out.println("    ** Invalid Addresses");
	    if (invalid != null) {
	      for (i = 0; i < invalid.length; i++) 
		System.out.println("         " + invalid[i]);
	    }
	  }
	  Address[] validUnsent = sfex.getValidUnsentAddresses();
	  if (validUnsent != null) {
	    System.out.println("    ** ValidUnsent Addresses");
	    if (validUnsent != null) {
	      for (i = 0; i < validUnsent.length; i++) 
		System.out.println("         "+validUnsent[i]);
	    }
	  }
	  Address[] validSent = sfex.getValidSentAddresses();
	  if (validSent != null) {
	    System.out.println("    ** ValidSent Addresses");
	    if (validSent != null) {
	      for (i = 0; i < validSent.length; i++) 
		System.out.println("         "+validSent[i]);
	    }
	  }
	}
	System.out.println();
	if (ex instanceof MessagingException)
	  ex = ((MessagingException)ex).getNextException();
	else
	  ex = null;
      } while (ex != null);
    }
  }

  private DataHandler getChangeData(Change chng) {
    Object[] args_change = { new Integer(chng.getNumber()) };
    Object[] args_user = { chng.getUser().getId(),
			   chng.getUser().getEmail(),
			   chng.getUser().getFullName() };
    String chng_fmt = env.getProperty("p4.reviewer.url.change");

    String result;
    StringBuffer sb = new StringBuffer("<HTML><BODY>");

    sb.append("Change ");
    if (null == chng_fmt) {
      sb.append(chng.getNumber());
    } else {
      result = MessageFormat.format("<A href=\""+chng_fmt+
				    "\">{0,number,#}</A>", args_change);
      sb.append(result);
    }
    sb.append(" by ");
    result = MessageFormat.format("<A href=\"mailto:{1}\">{2}</A>",args_user);
    sb.append(result);
    sb.append(" on ");
    sb.append(chng.getModtimeString());
    sb.append("<P>");
    sb.append(chng.getShortDescription(true));
    sb.append("<P>Affected Files...<P><PRE>");
    Enumeration en = chng.getFiles().elements();
    while (en.hasMoreElements()) {
      sb.append("\t");
      sb.append(en.nextElement());
      sb.append("\n");
    }
    sb.append("</PRE>");
    sb.append(env.getProperty("p4.reviewer.sig.html","<HR>P4 Reviewer"));
    sb.append("</BODY></HTML>");
    return new DataHandler(new StringDataSource(sb.toString(),"text/html"));
  }
}
